@@ -88,8 +88,15 @@ sub new {
       $proto = $_[0][0];
       $port  = $_[0][1];
     }
+    elsif ($_[0] =~ /^(\d+)\D(\d+)$/) {
+      ($proto, $port) = ($1, $2);
+    }
+    elsif ($_[0] =~ /^\d+$/) {
+      $proto = ($_[0] & 0xf0000) >> 16;
+      $port  = $_[0] & 0x0ffff;
+    }
     else {
-      ($proto, $port) = $_[0] =~ /^(\d+)\D+(\d+)$/;
+      croak "invalid proto/port: $_[0]";
     }
   }
   elsif (@_) {
@@ -97,7 +104,8 @@ sub new {
     $port  = shift;
   }
   defined $proto && defined $port
-    || croak("invalid proto/port: array ref, int pair, or colon-separated str required");
+    || croak("invalid proto/port: array ref, int pair," .
+             " int, or colon-separated str required");
   $class->init($proto, $port);
 }
 
@@ -120,12 +128,12 @@ sub copy {
 
 sub str {
   my $self = shift;
-  sprintf("%03d:%05d", $self->proto, $self->port);
+  join(':', $self->proto, $self->port);
 }
 
 sub _cmp {
   my($self, $other, $rev) = @_;
-  $other = $self->new(@$other) unless UNIVERSAL::isa($other, __PACKAGE__);
+  $other = (ref $self)->new($other) unless UNIVERSAL::isa($other, __PACKAGE__);
   ($self, $other) = ($other, $self) if $rev;
   $self->[0] <=> $other->[0] || $self->[1] <=> $other->[1];
 }
@@ -139,20 +147,33 @@ sub _eq  {    defined $_[1]  && shift->_cmp(@_) == 0 }
 sub _ne  { (! defined $_[1]) || shift->_cmp(@_) != 0 }
 
 sub _add {
-  my($l, $r, $rev) = @_;
-  $l->_xs_add($r);
-  $l;
+  my($self, $num, $rev) = @_;
+  my $new = $self->_xs_add($num);
+  $$self = $$new;
 }
 
-sub _copy_add { shift->copy->_add(@_) }
+sub _copy_add {
+  my $self = shift;
+  $self->_xs_add(shift);
+}
 
 sub _sub {
-  my($l, $r, $rev) = @_;
-  $l->_xs_sub($r);
-  $l;
+  my($self, $num, $rev) = @_;
+  my $new = $self->_xs_sub($num);
+  $$self = $$new;
 }
 
-sub _copy_sub { shift->copy->_sub(@_) }
+sub _copy_sub {
+  my $self = shift;
+  $self->xs_sub(shift);
+}
+
+sub distance {
+  my($self, $other, $rev) = @_;
+  $other = (ref $self)->new($other);
+  $self >= $other ? $self->num - $other->num
+                  : $other->num - $self->num;
+}
 
 ###
 
@@ -249,11 +270,11 @@ containing the pair, or as a string with a non-digit separator such as
 
 =item proto()
 
-Return the protocol number.
+Return or set the protocol number.
 
 =item port()
 
-Return the port number.
+Return or set the port number.
 
 =item copy()
 
@@ -264,6 +285,14 @@ Return a copy of this object. This method is tied to the C<=> operator.
 Return a string representation "port:proto". This method is tied to the
 C<""> operator.
 
+=item num()
+Return an integer representation of this proto/port pair.
+
+=item distance($other)
+
+Return an integer representing the absolute value of the distance
+between this protoport and another.
+
 =back
 
 =head1 TIED ARRAY
@@ -291,11 +320,11 @@ C<Net::Silk::ProtoPort> objects:
 
 =head1 SEE ALSO
 
-L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::IPWildcard>, L<Net::Silk::Bag>, L<Net::Silk::Pmap>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
+L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::Bag>, L<Net::Silk::Pmap>, L<Net::Silk::IPWildcard>, L<Net::Silk::Range>, L<Net::Silk::CIDR>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (C) 2011-2015 by Carnegie Mellon University
+Copyright (C) 2011-2016 by Carnegie Mellon University
 
 Use of the Net-Silk library and related source code is subject to the
 terms of the following licenses:
