@@ -42,7 +42,7 @@
  * Convert a string to an int128_t/uint128_t.
  */
 
-static uint128_t
+static groknum_err_t
 strtoint128(pTHX_ const char *s, STRLEN len,
                   int base, int sign, uint128_t *u128) {
     uint128_t acc = 0;
